@@ -1,4 +1,4 @@
-from ossapi import OssapiV2, Ossapi
+from ossapi import OssapiV2, Ossapi, Grant
 
 import os
 
@@ -12,5 +12,6 @@ if not client_secret:
     client_secret = input("Enter your api v2 client secret: ")
 
 client_id = int(client_id)
-api = OssapiV2(client_id, client_secret, strict=True)
+api = OssapiV2(client_id, client_secret, strict=True,
+    grant=Grant.CLIENT_CREDENTIALS)
 apiv1 = Ossapi(key)
