@@ -33,17 +33,21 @@ class TestBeatmap(TestCase):
     def test_deserialize(self):
         api.beatmap(beatmap_id=221777)
 
+class TestBeatmapset(TestCase):
+    def test_deserialize(self):
+        api.beatmapset(beatmap_id=3207950)
+
 class TestBeatmapsetEvents(TestCase):
     def test_deserialize(self):
-        api.beatmapsets_events()
+        api.beatmapset_events()
 
     def test_all_types(self):
-        # beatmapsets_events is a really complicated endpoint in terms of return
+        # beatmapset_events is a really complicated endpoint in terms of return
         # types. We want to make sure both that we're not doing anything wrong,
         # and the osu! api isn't doing anything wrong by returning something
         # that doesn't match their documentation.
         for event_type in BeatmapsetEventType:
-            api.beatmapsets_events(types=[event_type])
+            api.beatmapset_events(types=[event_type])
 
 class TestRanking(TestCase):
     def test_deserialize(self):
@@ -121,8 +125,25 @@ class TestBeatmapsetDiscussions(TestCase):
     def test_deserialize(self):
         api.beatmapset_discussions()
 
-# TODO revisit when either ``api`` is scoped differently (requires redirect_uri
-# stored) or auth handling is rewriting to allow multiple clients
+class TestNewsListing(TestCase):
+    def test_deserialize(self):
+        api.news_listing(year=2021)
+
+class TestNewsPost(TestCase):
+    def test_deserialize(self):
+        api.news_post(1025, key="id")
+
+class TestSeasonalBackgrounds(TestCase):
+    def test_deserialize(self):
+        api.seasonal_backgrounds()
+
+
+# TODO requires friends.read scope
+# class TestFriends(TestCase):
+#     def test_deserialize(self):
+#         api.friends()
+
+# TODO requires chat.write scope
 # class TestCreateNewPM(TestCase):
 #     def test_deserialize(self):
 #         api.create_pm(2070907, "Unit test from ossapi "
